@@ -11,8 +11,11 @@ use std::io::BufReader;
 use std::path::Path;
 use std::sync::Arc;
 use std::thread::{self, JoinHandle};
+#[cfg(feature = "vad")]
 use std::time::SystemTime;
 static INTERNAL_SAMPLE_RATE: usize = 48000;
+
+#[cfg(feature = "vad")]
 /// Allowed vad modes
 pub type VadMode = webrtc_vad::VadMode;
 /// Allowed wav sample formats
@@ -30,8 +33,12 @@ pub struct WakewordDetectorBuilder {
     sample_rate: Option<usize>,
     sample_format: Option<SampleFormat>,
     bits_per_sample: Option<u16>,
+    channels: Option<u16>,
+    #[cfg(feature = "vad")]
     vad_mode: Option<VadMode>,
+    #[cfg(feature = "vad")]
     vad_sensitivity: Option<f32>,
+    #[cfg(feature = "vad")]
     vad_delay: Option<u16>,
     eager_mode: bool,
     single_thread: bool,
@@ -47,11 +54,15 @@ impl WakewordDetectorBuilder {
             sample_rate: None,
             sample_format: None,
             bits_per_sample: None,
+            channels: None,
             // detection options
             eager_mode: false,
             single_thread: false,
+            #[cfg(feature = "vad")]
             vad_mode: None,
+            #[cfg(feature = "vad")]
             vad_delay: None,
+            #[cfg(feature = "vad")]
             vad_sensitivity: None,
             threshold: None,
             averaged_threshold: None,
@@ -65,15 +76,19 @@ impl WakewordDetectorBuilder {
             self.get_sample_rate(),
             self.get_sample_format(),
             self.get_bits_per_sample(),
+            self.get_channels(),
             self.get_eager_mode(),
             self.get_single_thread(),
-            self.get_vad_mode(),
-            self.get_vad_delay(),
-            self.get_vad_sensitivity(),
             self.get_threshold(),
             self.get_averaged_threshold(),
             self.get_comparator_band_size(),
             self.get_comparator_ref(),
+            #[cfg(feature = "vad")]
+            self.get_vad_mode(),
+            #[cfg(feature = "vad")]
+            self.get_vad_delay(),
+            #[cfg(feature = "vad")]
+            self.get_vad_sensitivity(),
         )
     }
     /// Configures the detector threshold,
@@ -120,6 +135,14 @@ impl WakewordDetectorBuilder {
         self.sample_format = Some(value);
         self
     }
+    /// Configures the detector expected number of channels for the audio chunks to process.
+    /// Rustpotter will only use data for first channel.
+    ///
+    /// Defaults to 1
+    pub fn set_channels(&mut self, value: u16) -> &mut Self {
+        self.channels = Some(value);
+        self
+    }
     /// Configures the band-size for the comparator used to match the samples.
     ///
     /// Defaults to 6
@@ -155,6 +178,7 @@ impl WakewordDetectorBuilder {
         self.single_thread = value;
         self
     }
+    #[cfg(feature = "vad")]
     /// Seconds to disable the vad detector after voice is detected.
     ///
     /// Defaults to 3.
@@ -164,6 +188,7 @@ impl WakewordDetectorBuilder {
         self.vad_delay = Some(value);
         self
     }
+    #[cfg(feature = "vad")]
     /// Voice/silence ratio in the last second to consider voice detected.
     ///
     /// Defaults to 0.5.
@@ -174,6 +199,7 @@ impl WakewordDetectorBuilder {
         self.vad_sensitivity = Some(value);
         self
     }
+    #[cfg(feature = "vad")]
     /// Use a vad detector to reduce computation on absence of voice sound.
     ///
     /// Unless specified the vad detector is disabled.
@@ -196,6 +222,9 @@ impl WakewordDetectorBuilder {
     fn get_bits_per_sample(&self) -> u16 {
         self.bits_per_sample.unwrap_or(16)
     }
+    fn get_channels(&self) -> u16 {
+        self.channels.unwrap_or(1)
+    }
     fn get_comparator_band_size(&self) -> usize {
         self.comparator_band_size.unwrap_or(11)
     }
@@ -208,12 +237,15 @@ impl WakewordDetectorBuilder {
     fn get_single_thread(&self) -> bool {
         self.single_thread
     }
+    #[cfg(feature = "vad")]
     fn get_vad_sensitivity(&self) -> f32 {
         self.vad_sensitivity.unwrap_or(0.5)
     }
+    #[cfg(feature = "vad")]
     fn get_vad_delay(&self) -> u16 {
         self.vad_delay.unwrap_or(3)
     }
+    #[cfg(feature = "vad")]
     fn get_vad_mode(&self) -> Option<VadMode> {
         if self.vad_mode.is_none() {
             return None;
@@ -250,6 +282,7 @@ pub struct WakewordDetector {
     // input options
     sample_format: SampleFormat,
     bits_per_sample: u16,
+    channels: u16,
     // detection options
     threshold: f32,
     averaged_threshold: f32,
@@ -257,8 +290,11 @@ pub struct WakewordDetector {
     single_thread: bool,
     resampler: Option<FftFixedInOut<f32>>,
     comparator: Arc<FeatureComparator>,
+    #[cfg(feature = "vad")]
     vad_detector: Option<webrtc_vad::Vad>,
+    #[cfg(feature = "vad")]
     vad_delay: u16,
+    #[cfg(feature = "vad")]
     vad_sensitivity: f32,
     // state
     samples_per_frame: usize,
@@ -270,9 +306,13 @@ pub struct WakewordDetector {
     result_state: Option<DetectedWakeword>,
     extractor: DenoiseFeatures,
     resampler_out_buffer: Option<Vec<Vec<f32>>>,
+    #[cfg(feature = "vad")]
     vad_enabled: bool,
+    #[cfg(feature = "vad")]
     voice_detections: Vec<bool>,
+    #[cfg(feature = "vad")]
     voice_detection_time: SystemTime,
+    #[cfg(feature = "vad")]
     audio_cache: Vec<Vec<f32>>,
 }
 impl WakewordDetector {
@@ -284,27 +324,29 @@ impl WakewordDetector {
         sample_rate: usize,
         sample_format: SampleFormat,
         bits_per_sample: u16,
+        channels: u16,
         // detection options
         eager_mode: bool,
         single_thread: bool,
-        vad_mode: Option<VadMode>,
-        vad_delay: u16,
-        vad_sensitivity: f32,
         threshold: f32,
         averaged_threshold: f32,
         comparator_band_size: usize,
         comparator_ref: f32,
+        #[cfg(feature = "vad")] vad_mode: Option<VadMode>,
+        #[cfg(feature = "vad")] vad_delay: u16,
+        #[cfg(feature = "vad")] vad_sensitivity: f32,
     ) -> Self {
-        let mut samples_per_frame = 480;
+        let mut samples_per_frame = 480 * channels as usize;
         let resampler = if sample_rate != INTERNAL_SAMPLE_RATE {
             let resampler =
                 FftFixedInOut::<f32>::new(sample_rate, INTERNAL_SAMPLE_RATE, samples_per_frame, 1)
                     .unwrap();
-            samples_per_frame = resampler.input_frames_next();
+            samples_per_frame = resampler.input_frames_next() * channels as usize;
             Some(resampler)
         } else {
             None
         };
+        #[cfg(feature = "vad")]
         let vad_detector = if vad_mode.is_some() {
             Some(webrtc_vad::Vad::new_with_rate_and_mode(
                 webrtc_vad::SampleRate::Rate48kHz,
@@ -319,6 +361,7 @@ impl WakewordDetector {
             sample_format,
             bits_per_sample,
             samples_per_frame,
+            channels,
             frames: Vec::new(),
             wakewords: HashMap::new(),
             buffering: true,
@@ -335,12 +378,19 @@ impl WakewordDetector {
             resampler,
             eager_mode,
             single_thread,
+            #[cfg(feature = "vad")]
             voice_detections: Vec::with_capacity(100),
+            #[cfg(feature = "vad")]
             audio_cache: Vec::with_capacity(100),
+            #[cfg(feature = "vad")]
             vad_enabled: false,
+            #[cfg(feature = "vad")]
             vad_detector,
+            #[cfg(feature = "vad")]
             vad_delay,
+            #[cfg(feature = "vad")]
             vad_sensitivity,
+            #[cfg(feature = "vad")]
             voice_detection_time: SystemTime::UNIX_EPOCH,
         };
         detector
@@ -374,7 +424,7 @@ impl WakewordDetector {
         save_file(path, 0, &model).or(Err(String::from("Unable to generate file")))
     }
     /// Adds a wakeword using wav samples.
-    /// 
+    ///
     /// ```
     /// let mut word_detector = detector_builder.build();
     /// word_detector.add_wakeword(
@@ -427,29 +477,29 @@ impl WakewordDetector {
         self.update_detection_frame_size(min_frames, max_frames);
     }
     /// Process i32 audio chunks.
-    /// 
+    ///
     /// Asserts that the audio chunk length should match the return
     /// of the get_samples_per_frame method.
-    /// 
+    ///
     /// Assumes sample rate match the configured for the detector.
-    /// 
+    ///
     /// Asserts that detector bits_per_sample is one of: 8, 16, 24, 32.
-    /// 
+    ///
     /// Asserts that detector sample_format is 'int'.
-    /// 
+    ///
     /// It's an alias for the process_i32 method.
     pub fn process(&mut self, audio_chunk: &[i32]) -> Option<DetectedWakeword> {
         self.process_i32(audio_chunk)
     }
     /// Process i8 audio chunks.
-    /// 
+    ///
     /// Asserts that the audio chunk length should match the return
     /// of the get_samples_per_frame method.
-    /// 
+    ///
     /// Assumes sample rate match the configured for the detector.
-    /// 
+    ///
     /// Asserts that detector bits_per_sample is 8.
-    /// 
+    ///
     /// Asserts that detector sample_format is 'int'.
     pub fn process_i8(&mut self, audio_chunk: &[i8]) -> Option<DetectedWakeword> {
         assert!(self.bits_per_sample == 8);
@@ -461,14 +511,14 @@ impl WakewordDetector {
         )
     }
     /// Process i16 audio chunks.
-    /// 
+    ///
     /// Asserts that the audio chunk length should match the return
     /// of the get_samples_per_frame method.
-    /// 
+    ///
     /// Assumes sample rate match the configured for the detector.
-    /// 
+    ///
     /// Asserts that detector bits_per_sample is one of: 8, 16.
-    /// 
+    ///
     /// Asserts that detector sample_format is 'int'.
     pub fn process_i16(&mut self, audio_chunk: &[i16]) -> Option<DetectedWakeword> {
         assert!(self.bits_per_sample == 8 || self.bits_per_sample == 16);
@@ -480,14 +530,14 @@ impl WakewordDetector {
         )
     }
     /// Process i32 audio chunks.
-    /// 
+    ///
     /// Asserts that the audio chunk length should match the return
     /// of the get_samples_per_frame method.
-    /// 
+    ///
     /// Assumes sample rate match the configured for the detector.
-    /// 
+    ///
     /// Asserts that detector bits_per_sample is one of: 8, 16, 24, 32.
-    /// 
+    ///
     /// Asserts that detector sample_format is 'int'.
     pub fn process_i32(&mut self, audio_chunk: &[i32]) -> Option<DetectedWakeword> {
         assert!(
@@ -499,23 +549,31 @@ impl WakewordDetector {
         self.process_int(audio_chunk)
     }
     /// Process f32 audio chunks.
-    /// 
+    ///
     /// Asserts that the audio chunk length should match the return
     /// of the get_samples_per_frame method.
-    /// 
+    ///
     /// Assumes sample rate match the configured for the detector.
-    /// 
+    ///
     /// Asserts that detector bits_per_sample is 32.
-    /// 
+    ///
     /// Asserts that detector sample_format is 'float'.
     pub fn process_f32(&mut self, audio_chunk: &[f32]) -> Option<DetectedWakeword> {
         assert!(audio_chunk.len() == self.samples_per_frame);
         assert!(self.bits_per_sample == 32);
         assert!(self.sample_format == SampleFormat::Float);
-        let float_buffer: Vec<f32> = audio_chunk
-            .into_iter()
-            .map(|s| s * 32767.0)
-            .collect::<Vec<_>>();
+        let float_buffer: Vec<f32> = if self.channels != 1 {
+            audio_chunk
+                .chunks_exact(self.channels as usize)
+                .map(|chunk| chunk[0])
+                .map(|s| s * 32767.0)
+                .collect::<Vec<_>>()
+        } else {
+            audio_chunk
+                .into_iter()
+                .map(|s| s * 32767.0)
+                .collect::<Vec<_>>()
+        };
         let resampled_audio = if self.resampler.is_some() {
             let resampler = self.resampler.as_mut().unwrap();
             let waves_in = vec![float_buffer; 1];
@@ -528,7 +586,10 @@ impl WakewordDetector {
         } else {
             float_buffer
         };
-        self.apply_vad_detection(resampled_audio)
+        #[cfg(not(feature = "vad"))]
+        return self.process_encoded_audio(&resampled_audio, true);
+        #[cfg(feature = "vad")]
+        return self.apply_vad_detection(resampled_audio);
     }
     /// Returns the desired chunk size.
     pub fn get_samples_per_frame(&self) -> usize {
@@ -570,16 +631,30 @@ impl WakewordDetector {
         let resampled_audio = if self.resampler.is_some() {
             let resampler = self.resampler.as_mut().unwrap();
             let bits_per_sample = self.bits_per_sample;
-            let float_buffer: Vec<f32> = audio_chunk
-                .into_iter()
-                .map(move |s| {
-                    if bits_per_sample < 16 {
-                        (*s << (16 - bits_per_sample)) as f32
-                    } else {
-                        (*s >> (bits_per_sample - 16)) as f32
-                    }
-                })
-                .collect::<Vec<_>>();
+            let float_buffer = if self.channels != 1 {
+                audio_chunk
+                    .chunks_exact(self.channels as usize)
+                    .map(|chunk| chunk[0])
+                    .map(|s| {
+                        if bits_per_sample < 16 {
+                            (s << (16 - bits_per_sample)) as f32
+                        } else {
+                            (s >> (bits_per_sample - 16)) as f32
+                        }
+                    })
+                    .collect::<Vec<_>>()
+            } else {
+                audio_chunk
+                    .into_iter()
+                    .map(|s| {
+                        if bits_per_sample < 16 {
+                            (*s << (16 - bits_per_sample)) as f32
+                        } else {
+                            (*s >> (bits_per_sample - 16)) as f32
+                        }
+                    })
+                    .collect::<Vec<_>>()
+            };
             let waves_in = vec![float_buffer; 1];
             let waves_out = self.resampler_out_buffer.as_mut().unwrap();
             resampler
@@ -593,8 +668,12 @@ impl WakewordDetector {
                 .map(|n| *n as f32)
                 .collect::<Vec<_>>()
         };
-        self.apply_vad_detection(resampled_audio)
+        #[cfg(not(feature = "vad"))]
+        return self.process_encoded_audio(&resampled_audio, true);
+        #[cfg(feature = "vad")]
+        return self.apply_vad_detection(resampled_audio);
     }
+    #[cfg(feature = "vad")]
     fn apply_vad_detection(&mut self, resampled_audio: Vec<f32>) -> Option<DetectedWakeword> {
         if self.result_state.is_none()
             && self.vad_detector.is_some()
@@ -685,24 +764,21 @@ impl WakewordDetector {
         let sample_format = wav_reader.spec().sample_format;
         let bits_per_sample = wav_reader.spec().bits_per_sample;
         let channels = wav_reader.spec().channels;
-        if channels != 1 {
-            return Err("Only samples with 1 channels are supported for now".to_string());
-        }
         let audio_pcm_signed_resampled = match sample_format {
             SampleFormat::Int => {
                 let bits_per_sample = bits_per_sample;
                 assert!(bits_per_sample <= 32);
                 let samples = wav_reader
                     .into_samples::<i32>()
-                    .map(move |s| {
-                        s.map(|s| {
-                            if bits_per_sample < 16 {
-                                (s << (16 - bits_per_sample)) as f32
-                            } else {
-                                (s >> (bits_per_sample - 16)) as f32
-                            }
-                        })
-                        .unwrap()
+                    .collect::<Vec<_>>()
+                    .chunks_exact(channels as usize)
+                    .map(|chunk| chunk[0].as_ref().unwrap())
+                    .map(|s| {
+                        if bits_per_sample < 16 {
+                            (*s << (16 - bits_per_sample)) as f32
+                        } else {
+                            (*s >> (bits_per_sample - 16)) as f32
+                        }
                     })
                     .collect::<Vec<_>>();
                 if sample_rate as usize != INTERNAL_SAMPLE_RATE {
@@ -714,7 +790,10 @@ impl WakewordDetector {
             SampleFormat::Float => {
                 let samples = wav_reader
                     .into_samples::<f32>()
-                    .map(|s| s.map(|s| s * 32767.0).unwrap())
+                    .collect::<Vec<_>>()
+                    .chunks_exact(channels as usize)
+                    .map(|chunk| chunk[0].as_ref().unwrap())
+                    .map(|s| *s * 32767.0)
                     .collect::<Vec<_>>();
                 if sample_rate as usize != INTERNAL_SAMPLE_RATE {
                     resample_audio(sample_rate as usize, &samples)
@@ -747,11 +826,14 @@ impl WakewordDetector {
         let result_option = self.get_best_wakeword(features);
         match result_option {
             Some(result) => {
-                if self.vad_enabled {
-                    self.vad_enabled = false;
-                    debug!("switching to feature detector");
+                #[cfg(feature = "vad")]
+                {
+                    if self.vad_enabled {
+                        self.vad_enabled = false;
+                        debug!("switching to feature detector");
+                    }
+                    self.voice_detection_time = SystemTime::now();
                 }
-                self.voice_detection_time = SystemTime::now();
                 if self.eager_mode {
                     if result.index != 0 {
                         debug!("Sorting '{}' templates", result.wakeword);
@@ -808,7 +890,9 @@ impl WakewordDetector {
         }
     }
     fn reset(&mut self) {
-        self.voice_detection_time = SystemTime::now();
+        #[cfg(feature = "vad")] {
+            self.voice_detection_time = SystemTime::now();
+        }
         self.frames.clear();
         self.buffering = true;
         self.result_state = None;
